/*
 * The Alluxio Open Foundation licenses this work under the Apache License, version 2.0
 * (the "License"). You may not use this work except in compliance with the License, which is
 * available at www.apache.org/licenses/LICENSE-2.0
 *
 * This software is distributed on an "AS IS" basis, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied, as more fully set forth in the License.
 *
 * See the NOTICE file distributed with this work for information regarding copyright ownership.
 */

provider "aws" {
  region  = var.compute_region
  version = "~> 2.56"
}

resource "random_id" "name_suffix" {
  byte_length = 4
}

locals {
  presuffix_name = var.name == "" ? random_id.name_suffix.hex : var.name
  name_prefix    = var.randomize_name ? "${local.presuffix_name}_${random_id.name_suffix.hex}" : local.presuffix_name

  compute_name_prefix = "${local.name_prefix}-compute"

  shared_bucket_name_replaced = replace("shared-${local.name_prefix}-bucket", "_", "-") // underscores are not allowed
  shared_bucket_name_lowered  = lower(local.shared_bucket_name_replaced)                // uppercase letters not allowed
}

resource "aws_s3_bucket" "shared_s3_bucket" {
  // the shared s3 bucket can be in any region
  // make sure the objects that upload to the bucket belong to the aws provider of the bucket region
  // bucket in compute region can provide better performance
  bucket        = local.shared_bucket_name_lowered
  force_destroy = true
}

// compute cluster resources
locals {
  compute_master_instance_type = "r4.4xlarge"
  compute_worker_instance_type = "r5d.4xlarge"
  compute_instance_types       = [local.compute_master_instance_type, local.compute_worker_instance_type]
}

module "vpc_compute" {                                                      
  source = "./alluxio/vpc_with_internet/aws"
  aws_availability_zone_blacklist = var.availability_zone_blacklist
  aws_instance_types              = local.compute_instance_types
}

module "security_group_compute" {                                                
  source = "./alluxio/alluxio_security_group/aws"
  name                            = local.compute_name_prefix
  randomize_name                  = var.randomize_name
  vpc_id                          = module.vpc_compute.vpc_id
  alluxio_web_ui_rule_cidr_blocks = var.local_ip_as_cidr != "" ? [var.local_ip_as_cidr] : []
}

module "aws_key_pair_compute" {
  source  = "terraform-aws-modules/key-pair/aws"
  version = "0.6.0"

  create_key_pair = var.compute_key_pair_name == null
  key_name        = var.compute_key_pair_name == null ? "${local.compute_name_prefix}-keypair" : var.compute_key_pair_name
  public_key      = file(var.ssh_public_key_file_path)
}

resource "aws_s3_bucket_object" "compute_alluxio_bootstrap" {
  bucket = aws_s3_bucket.shared_s3_bucket.bucket
  key    = "alluxio-emr.sh"
  source = "emr/alluxio-emr.sh"
}

resource "aws_s3_bucket_object" "compute_presto_bootstrap" {
  bucket = aws_s3_bucket.shared_s3_bucket.bucket
  key    = "presto-emr.sh"
  source = "emr/presto-emr.sh"
}

module "alluxio_compute" {
  source = "./alluxio/alluxio_cloud_cluster/aws"

  name                          = "${local.compute_name_prefix}-compute"
  randomize_name                = var.randomize_name
  aws_security_group_id         = module.security_group_compute.emr_managed_sg_id
  additional_security_group_ids = [module.security_group_compute.alluxio_sg_id]
  aws_subnet_id                 = module.vpc_compute.subnet_id
  aws_key_pair_name             = module.aws_key_pair_compute.this_key_pair_key_name

  alluxio_tarball_url      = var.alluxio_tarball_url
  alluxio_working_bucket   = aws_s3_bucket.shared_s3_bucket.bucket
  alluxio_bootstrap_s3_uri = "s3://${aws_s3_bucket.shared_s3_bucket.bucket}/${aws_s3_bucket_object.compute_alluxio_bootstrap.key}"
  presto_bootstrap_s3_uri  = "s3://${aws_s3_bucket.shared_s3_bucket.bucket}/${aws_s3_bucket_object.compute_presto_bootstrap.key}"

  master_config = {
    ebs_volume_size = 32
    instance_count  = 1
    instance_type   = local.compute_master_instance_type
  }
  worker_config = {
    ebs_volume_size = 32
    instance_count  = 3
    instance_type   = local.compute_worker_instance_type
  }
  alluxio_nvme_percentage = 80
}
